--- sock.h ---
#ifndef SOCK_H
#define SOCK_H


/* 
 * Simple, portable socket and POP3 implentation by Laura Michaels
 * This code is made available in accordance with the terms and 
 * conditions of the GNU Library General Public License.  Contact the
 * Free Software Foundation or see http://www.gnu.org/ if you
 * need information on GNU licensing conditions.
 */


#ifdef UNIX
#define SOCKET int
#define CPPSOCKET_INVALID -1
//If the Unix system doesn't supply socklen_t, define it as int as below.
#else
#define CPPSOCKET_INVALID INVALID_SOCKET
#define socklen_t int
#endif

#define CPPSOCKET_ERROR -1
#define CPPSOCKET_TIMEOUT 2
#define CPPSOCKET_BUFFER_OVERFLOW 3
#define CPPSOCKET_NAK 4




/*
 * Class to provide basic, portable, BSD socket implementation.
 * Supplies most of the socket functions for implementing both
 * servers and clients.
 */
class lmsocket 
{
public:
   lmsocket ();
   ~lmsocket ();
/*
 * Name: 
 * lmsocket::create
 *
 * Purpose:
 * Creates a socket.
 *
 * Usage:
 * bool create (SOCKET &so, int sockdomain, int socktype, int sockprotocol)
 * 
 * Data:
 * Name          Use    Description
 * so             O     Socket descriptor
 * sockdomain    I      Address family
 * socktype      I      Socket type
 * sockprotocol  I      Socket protocol
 *                      (0 = automatically choose protocol)
 *
 * Return:
 * Error flag.
 * (true = socket created, false = socket not created)
 *
 * Description:
 * Calls the socket function to create a socket.
 *
 * History:
 * 19990925  LM  Created
 * 
 */
   bool create (SOCKET &so, int sockdomain, int socktype, int sockprotocol);
/*
 * Name: 
 * lmsocket::connect
 *
 * Purpose:
 * Requests connection between two sockets.
 *
 * Usage:
 * bool connect (SOCKET so, struct sockaddr &sa)
 * 
 * Data:
 * Name          Use    Description
 * so            I      Socket descriptor
 * sockaddr      I      Address of peer socket
 *
 * Return:
 * Error flag.
 * (true = successful, false = socket could not connect)
 *
 * Description:
 * Creates a connection to an unconnected socket.  Usually used by
 * a client program to request a connection to a server program through
 * sockets.  May be a blocking operation.
 *
 * History:
 * 19990925  LM  Created
 * 
 */
   bool connect (SOCKET so, struct sockaddr &sa);
/*
 * Name: 
 * lmsocket::bind
 *
 * Purpose:
 * Binds a name to a socket.
 *
 * Usage:
 * bool bind (SOCKET so, struct sockaddr &sa)
 * 
 * Data:
 * Name          Use    Description
 * so            I      Socket descriptor
 * sockaddr      I      Address of socket
 *
 * Return:
 * Error flag.
 * (true = successful, false = bind unsuccessful)
 *
 * Description:
 * Creates a named socket.  Used on an unconnected socket.
 * Usually called by a server program.
 *
 * History:
 * 19990925  LM  Created
 * 
 */
   bool bind (SOCKET so, struct sockaddr &sa);
/*
 * Name: 
 * lmsocket::listen
 *
 * Purpose:
 * Listens for socket connections.
 *
 * Usage:
 * bool listen (SOCKET so, int queuesize);
 * 
 * Data:
 * Name          Use    Description
 * so            I      Socket descriptor
 * queuesize     I      Number of pending connections allowed
 *                      (Maximum queue size = SOMAXCONN)
 *
 * Return:
 * Error flag.
 * (true = connection requested, false = connection failed)
 *
 * Description:
 * Listens for sockets that want to connect.  Limits the number of
 * pending connections in the queue.  Typically used by server
 * programs in handling socket connection requests.
 *
 * History:
 * 19990925  LM  Created
 * 
 */
   bool listen (SOCKET so, int queuesize);
/*
 * Name: 
 * lmsocket::accept
 *
 * Purpose:
 * Accepts a socket connection request.
 *
 * Usage:
 * bool accept (SOCKET &so, struct sockaddr &sa)
 * 
 * Data:
 * Name          Use    Description
 * so             O     Socket descriptor
 * sockaddr       O     Address of peer socket
 *
 * Return:
 * Error flag.
 * (true = connection created, false = connection failed)
 *
 * Description:
 * Creates a new socket connection.  Typically connects a server to a client
 * requesting a connection.  May be a blocking operation.
 *
 * History:
 * 19990925  LM  Created
 * 
 */
   bool accept (SOCKET &so, struct sockaddr &sa);
/*
 * Name: 
 * lmsocket::send
 *
 * Purpose:
 * Sends a message over a socket connection.
 *
 * Usage:
 * int send (SOCKET so, char *msg, int lnth = -1, int flags = 0)
 * 
 * Data:
 * Name          Use    Description
 * so            I      Socket descriptor
 * msg           I      Message
 * lnth          I      Length of message
 *                      (default or -1 = will calculate the message length)
 * flags         I      Set socket related parameters.  (See the
 *                      BSD socket send function documentation for valid
 *                      values.)
 *
 * Return:
 * Number of characters sent or CPPSOCKET_ERROR = error occurred
 *
 * Description:
 * Sends/writes a specified message using the BSD socket send function.
 * May be a blocking operation.
 *
 * History:
 * 19990925  LM  Created
 * 
 */
   int send (SOCKET so, char *msg, int lnth = -1, int flags = 0);
/*
 * Name: 
 * lmsocket::receive
 *
 * Purpose:
 * Receives a message over a socket connection.
 *
 * Usage:
 * int receive (SOCKET so, char *msg, int lnth, int flags = 0)
 * 
 * Data:
 * Name          Use    Description
 * so            I      Socket descriptor
 * msg            O     Message
 * lnth          I      Maximum size available in msg.
 * flags         I      Set socket related parameters.  (See the
 *                      BSD socket recv function documentation for valid
 *                      values.)
 *
 * Return:
 * Number of characters received,
 * 0 = connection closed or CPPSOCKET_ERROR = error occurred
 *
 * Description:
 * Receives/reads a message using the BSD socket recv function.
 * May be a blocking operation.
 *
 * History:
 * 19990925  LM  Created
 * 
 */
   int receive (SOCKET so, char *msg, int lnth, int flags = 0);
/*
 * Name: 
 * lmsocket::close
 *
 * Purpose:
 * Closes the socket.
 *
 * Usage:
 * bool close (SOCKET so)
 * 
 * Data:
 * Name          Use    Description
 * so            I      Socket descriptor
 *
 * Return:
 * Error flag.
 * (true = socket closed, false = problem closing socket)
 *
 * Description:
 * Closes socket and frees resources related to that socket.
 * May be a blocking operation.
 *
 * History:
 * 19990925  LM  Created
 * 
 */
   bool close (SOCKET so);
/*
 * Name: 
 * lmsocket::shutdown
 *
 * Purpose:
 * Disables send and/or receive operations on a socket.
 *
 * Usage:
 * bool shutdown (SOCKET so, int flags)
 * 
 * Data:
 * Name          Use    Description
 * so            I      Socket descriptor
 * flags         I      Set socket related parameters.
 *                      0 = disable receive, 1 = disable send
 *                      2 = disable send and receive
 *
 * Return:
 * Error flag.
 * (true = shutdown successful, false = shutdown failed)
 *
 * Description:
 * Can be used to inform connected socket that connection will be
 * closed.  To actually close the socket and free resources, close
 * must still be called.
 *
 * History:
 * 19990925  LM  Created
 * 
 */
   bool shutdown (SOCKET so, int flags);
/*
 * Name: 
 * lmsocket::isreadready
 *
 * Purpose:
 * Checks if specified descriptors are ready for reading.
 *
 * Usage:
 * int isreadready (bool *soready, SOCKET *so, int socnt, struct timeval *tve)
 * 
 * Data:
 * Name          Use    Description
 * soready        O     Array of flags indicating which descriptors are ready
 *                      for reading
 * so            I      Array of descriptors to check
 * socnt         I      Number or descriptors to check
 * tve           I      Timeout value
 *                      (tve = 0 blocks until a socket is available for reading,
 *                      tve.tv_sec = 0 and tve.tv_usec = 0 polls for a read
 *                      ready socket)
 *
 * Return:
 * Number of descriptors ready for reading,
 * 0 = timeout or CPPSOCKET_ERROR = error occurred
 *
 * Description:
 * Checks the status of connected sockets.  Waits until timeout
 * if no sockets are available for reading.  May be a blocking operation.
 *
 * History:
 * 19990925  LM  Created
 * 
 */
   int isreadready (bool *soready, SOCKET *so, int socnt, struct timeval *tve);
/*
 * Name: 
 * lmsocket::iswriteready
 *
 * Purpose:
 * Checks if specified descriptors are ready for writing.
 *
 * Usage:
 * int iswriteready (bool *soready, SOCKET *so, int socnt, struct timeval *tve)
 * 
 * Data:
 * Name          Use    Description
 * soready        O     Array of flags indicating which descriptors are ready
 *                      for writing
 * so            I      Array of descriptors to check
 * socnt         I      Number or descriptors to check
 * tve           I      Timeout value
 *                      (tve = 0 blocks until a socket is available for writing,
 *                      tve.tv_sec = 0 and tve.tv_usec = 0 polls for a write
 *                      ready socket)
 *
 * Return:
 * Number of descriptors ready for writing,
 * 0 = timeout or CPPSOCKET_ERROR = error occurred
 *
 * Description:
 * Checks the status of connected sockets.  Waits until timeout
 * if no sockets are available for writing.  May be a blocking operation.
 *
 * History:
 * 19990925  LM  Created
 * 
 */
   int iswriteready (bool *soready, SOCKET *so, int socnt, struct timeval *tve);
/*
 * Name: 
 * lmsocket::eval_error
 *
 * Purpose:
 * Gets an error value.
 *
 * Usage:
 * int eval_error (void)
 * 
 * Return:
 * Error value.
 *
 * Description:
 * Many socket functions return an error of -1 and set an error flag
 * that provides more information when they fail.  Returns the value of
 * the error flag.  Must be called just after a BSD socket function fails.
 *
 * History:
 * 19990925  LM  Created
 * 
 */
   int eval_error (void);
};


// Largest retrievable e-mail message size.
#define CPPSOCKET_BUFFER_SIZE 60000
#define CPPSOCKET_ENCRYPTION_BUFFER_SIZE 255


/*
 * Class to provide implementation of POP3 protocol.  Supplies most
 * of the commands for this protocol.  See RFC 1725 for more details
 * on POP3.
 */
class pop3_socket 
{
public:
   pop3_socket ();
   ~pop3_socket ();
/*
 * Name: 
 * lmsocket::open
 *
 * Purpose:
 * Creates and attempts to connect to a socket.
 *
 * Usage:
 * bool open (char *host)
 * 
 * Data:
 * Name          Use    Description
 * host          I      Host name for server to connect to.
 *
 * Return:
 * Error value.
 * (true = succeeded, false = failed)
 *
 * Description:
 * Sets the address.  Assumes the port is the default port for
 * POP3 transactions.
 *
 * History:
 * 19990925  LM  Created
 * 
 */
   bool open (char *host);
/*
 * Name: 
 * lmsocket::isencryptable
 *
 * Purpose:
 * Checks if password can be encrypted.
 *
 * Usage:
 * bool isencryptable (void)
 * 
 * Return:
 * Encrypt flag.
 * (true = can be encrypted, false = cannot be encrypted)
 *
 * Description:
 * Indicates whether a password can be encrypted before being sent
 * to the server.  This is based on information returned from the server
 * after the initial connection.  If false, the server will not understand
 * how to deal with an encrypted password.  If true, a password can be sent
 * using MD5 encryption.
 *
 * History:
 * 19990925  LM  Created
 * 
 */
   bool isencryptable (void);
/*
 * Name: 
 * lmsocket::setencrypt
 *
 * Purpose:
 * Tells whether to use encrypted or non-encrypted password.
 *
 * Usage:
 * void setencrypt (bool flag)
 * 
 * Data:
 * Name          Use    Description
 * flag          I      Encryption flag
 *                      (true = encrypt password,
 *                      false = do not encrypt password)
 *
 * Description:
 * Sets a flag to indicate whether the code should attempt to send
 * the password as encrypted or not when sending the password is required.
 *
 * History:
 * 19990925  LM  Created
 * 
 */
   void setencrypt (bool flag);
/*
 * Name: 
 * lmsocket::login
 *
 * Purpose:
 * Sends user and password information to the server for logging in.
 *
 * Usage:
 * bool login (char *user, char *password)
 * 
 * Data:
 * Name          Use    Description
 * user          I      User name
 * password      I      Password
 *
 * Return:
 * Error value.
 * (true = succeeded, false = failed)
 *
 * Description:
 * Sends user and password information to see if the server recognizes
 * a particular user and will allow connection to the user's e-mail.
 * Checks the encrypt flag (see setencrypt) to determine if the password
 * should be sent unencrypted or using MD5 encryption.
 *
 * History:
 * 19990925  LM  Created
 * 
 */
   bool login (char *user, char *password);
/*
 * Name: 
 * lmsocket::quit
 *
 * Purpose:
 * Tells the server the POP3 client is disconnecting.
 *
 * Usage:
 * bool quit (void)
 * 
 * Return:
 * Error value.
 * (true = succeeded, false = failed)
 *
 * Description:
 * Part of the POP3 protocol.  Sent to indicate when the client
 * is finished with its transactions.  Tells server to remove all
 * messages marked for deletion and release any locks.
 *
 * History:
 * 19990925  LM  Created
 * 
 */
   bool quit (void);
/*
 * Name: 
 * lmsocket::close
 *
 * Purpose:
 * Closes the client socket.
 *
 * Usage:
 * bool close (void)
 * 
 * Return:
 * Error value.
 * (true = succeeded, false = failed)
 *
 * Description:
 *
 * History:
 * 19990925  LM  Created
 * 
 */
   bool close (void);
/*
 * Name: 
 * lmsocket::stat
 *
 * Purpose:
 * Requests statistics from the server.
 *
 * Usage:
 * int stat (void)
 * 
 * Return:
 * Number of e-mails available
 *
 * Description:
 * Part of the POP3 protocol.  Requests information about the mail
 * system.  Parses out and returns the number of messages waiting.
 *
 * History:
 * 19990925  LM  Created
 * 
 */
   int stat (void);
/*
 * Name: 
 * lmsocket::noop
 *
 * Purpose:
 * No operation.
 *
 * Usage:
 * int stat (void)
 * 
 * Return:
 * Error value.
 * (true = succeeded, false = failed)
 *
 * Description:
 * Part of the POP3 protocol.  Does a noop.  Returns true
 * if server replies with a positive response.
 *
 * History:
 * 19990925  LM  Created
 * 
 */
   bool noop (void);
/*
 * Name: 
 * lmsocket::rset
 *
 * Purpose:
 * Undeletes messages.
 *
 * Usage:
 * bool rset (void)
 * 
 * Return:
 * Error value.
 * (true = succeeded, false = failed)
 *
 * Description:
 * Part of the POP3 protocol.  Any messages marked for deletion during
 * the connection are reset to an unmarked status so they will not be
 * deleted when the session ends (quit is sent).
 *
 * History:
 * 19990925  LM  Created
 * 
 */
   bool rset (void);
/*
 * Name: 
 * lmsocket::uidl
 *
 * Purpose:
 * Get a unique id listing.
 *
 * Usage:
 * bool uidl (char *msg, int lnth, int msgnum)
 *
 * Data:
 * Name          Use    Description
 * msg            O     Unique id
 * lnth          I      Size of msg buffer
 * msgnum        I      Non-deleted message number
 * 
 * Return:
 * Error value.
 * (true = succeeded, false = failed)
 *
 * Description:
 * Part of the POP3 protocol.  Requests a unique id listing for the
 * indicated non-deleted message number.  Parses and returns the
 * unique id for the corresponding message.  A unique id is a server
 * generated string of characters in the range between 0x21 and 0x7e
 * which uniquely identifies a message.  It can be used between
 * client-server sessions.
 * The uidl command can also be used without specifying a message number
 * and will then return unique id listings of all non-deleted
 * messages.  Using uidl without a message number is not currently
 * implemented in this class.
 *
 * History:
 * 19990925  LM  Created
 * 
 */
   bool uidl (char *msg, int lnth, int msgnum);
/*
 * Name: 
 * lmsocket::list
 *
 * Purpose:
 * Returns a scan listing for a specified message.
 *
 * Usage:
 * int list (int msgnum)
 *
 * Data:
 * Name          Use    Description
 * msgnum        I      Non-deleted message number
 * 
 * Return:
 * Message size in octets.
 *
 * Description:
 * Part of the POP3 protocol.  Requests a scan listing for the
 * indicated non-deleted message number.  Parses and returns the
 * size of the corresponding message in octets.
 * The list command can also be used without specifying a message
 * number and will then return the scan lines (including sizes in octets)
 * of all non-deleted messages.  Using list without a message number is
 * not currently implemented in this class.
 *
 * History:
 * 19990925  LM  Created
 * 
 */
   int list (int msgnum);
/*
 * Name: 
 * lmsocket::retr
 *
 * Purpose:
 * Retrieve a message.
 *
 * Usage:
 * int retr (char *msg, int lnth, int msgnum)
 *
 * Data:
 * Name          Use    Description
 * msg            O     Message
 * lnth          I      Size of msg buffer.
 * msgnum        I      Non-deleted message number
 * 
 * Return:
 * Error flag
 * (0 = no error, not equal 0 = error)
 *
 * Description:
 * Part of the POP3 protocol.  Retrieves specified e-mail message.
 * Parses out the server 'OK' response and the '.' (appearing on a newline)
 * which indicates the end of the message.  Places all or up to lnth of
 * the message into msg.  Check CPPSOCKET defines for more information on
 * error codes.
 *
 * History:
 * 19990925  LM  Created
 * 
 */
   int retr (char *msg, int lnth, int msgnum);
/*
 * Name: 
 * lmsocket::dele
 *
 * Purpose:
 * Deletes a message.
 *
 * Usage:
 * bool dele (int msgnum)
 *
 * Data:
 * Name          Use    Description
 * msgnum        I      Non-deleted message number
 * 
 * Return:
 * Error value.
 * (true = succeeded, false = failed)
 *
 * Description:
 * Part of the POP3 protocol.  Marks a message for deletion.
 * Messages are not actually deleted until a quit is sent to the server.
 *
 * History:
 * 19990925  LM  Created
 * 
 */
   bool dele (int msgnum);
private:
   bool userpassword (char *user, char *password);
// APOP uses MD5 encryption when sending a password.  The MD5 routine
// is not actually included in this class.  Sample MD5 encryption
// implementation in C is available as part of RFC 1321.  (See Appendix
// A of RFC 1321.)   
   bool apop (char *user, char *password);
   lmsocket sock;
   char buffer[CPPSOCKET_BUFFER_SIZE];
   char encservaddr[CPPSOCKET_ENCRYPTION_BUFFER_SIZE];
   bool encryptable;
   bool encryptit;
   SOCKET so;
   struct timeval tv;
};


#endif


--- sock.cpp ---
#ifdef UNIX
#include <sys/types.h>
#include <sys/time.h>
#include <sys/socket.h>
#include <sys/socketvar.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <netdb.h>
#include <unistd.h>
#include <strings.h>
#include <errno.h>
#include <memory.h>
#else
#include <winsock.h>
#include <mem.h>
#endif
#include <ctype.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <iostream.h>
#include "sock.h"


#ifdef UNIX
char *itoa(int value, char *string, int radix)
{
   sprintf (string, "%d", value);
   return (string);
}
#endif


lmsocket::lmsocket ()
{
#ifndef UNIX
   WSADATA wsaData;
   WSAStartup (0x101, &wsaData);
#endif
}

lmsocket::~lmsocket ()
{
#ifndef UNIX
   WSACleanup ();
#endif
}

bool lmsocket::create (SOCKET &so, int sockdomain, int socktype, int sockprotocol)
{
   bool retval;

   so = ::socket (sockdomain, socktype, sockprotocol);
   retval = (so != CPPSOCKET_INVALID) ? true : false;
   return (retval);
}

bool lmsocket::connect (SOCKET so, struct sockaddr &sa)
{
   bool retval;

   int cnt = ::connect (so, &sa, sizeof (sa));
   retval = (cnt != CPPSOCKET_ERROR) ? true : false;
   return (retval);
}

bool lmsocket::bind (SOCKET so, struct sockaddr &sa)
{
   bool retval;
   
   int cnt = ::bind (so, &sa, sizeof (sa));
   retval = (cnt != CPPSOCKET_ERROR) ? true : false;
   return (retval);
}

bool lmsocket::listen (SOCKET so, int queuesize)
{
   bool retval;
   int cnt = ::listen (so, queuesize);
   retval = (cnt != CPPSOCKET_ERROR) ? true : false;
   return (retval);
}

bool lmsocket::accept (SOCKET &so, struct sockaddr &sa)
{
   bool retval;
   socklen_t addrlnth = sizeof (sa);
   so = ::accept (so, &sa, &addrlnth);
   retval = (so != CPPSOCKET_INVALID) ? true : false;
   return (retval);
}

int lmsocket::send (SOCKET so, char *msg, int lnth, int flags)
{
   int strlnth;
   int lnthsent = 0;

   strlnth = (lnth == -1) ? strlen (msg) : lnth;
   lnthsent = ::send (so, msg, strlnth, flags);
   return (lnthsent);
}

int lmsocket::receive (SOCKET so, char *msg, int lnth, int flags)
{
   int lnthsent = recv (so, msg, lnth, flags);
   return (lnthsent);
}

bool lmsocket::shutdown (SOCKET so, int flags)
{
   bool retval;
   
   int cnt = ::shutdown (so, flags);
   retval = (cnt != CPPSOCKET_ERROR) ? true : false;
   return (retval);
}

bool lmsocket::close (SOCKET so)
{
   bool retval;
   int cnt;
#ifndef UNIX
   cnt = closesocket (so);
#else
   cnt = ::close (so);
#endif
   retval = (cnt != CPPSOCKET_ERROR) ? true : false;
   return (retval);
}

int lmsocket::isreadready (bool *soready, SOCKET *so, int socnt, struct timeval *tve)
{
   bool retval;
   int result;
   int x; 
   fd_set fds;

   FD_ZERO (&fds);
   for (x = 0; x < socnt; x++)
      FD_SET(so[x], &fds);
   result = select(so[socnt - 1] + 1, &fds, 0, 0, tve);
   for (x = 0; x < socnt; x++)
      if (FD_ISSET(so[x], &fds) != 0)
         soready[x] = true;
      else
         soready[x] = false;
   return (result);      
}

int lmsocket::iswriteready (bool *soready, SOCKET *so, int socnt, struct timeval *tve)
{
   bool retval;
   int result;
   int x; 
   fd_set fds;

   FD_ZERO (&fds);
   for (x = 0; x < socnt; x++)
      FD_SET(so[x], &fds);
   result = select(so[socnt - 1] + 1, 0, &fds, 0, tve);
   for (x = 0; x < socnt; x++)
      if (FD_ISSET(so[x], &fds) != 0)
         soready[x] = true;
      else
         soready[x] = false;
   return (result);      
}

int lmsocket::eval_error (void)
{
   int errflag;
#ifdef UNIX
   errflag = errno;
#else
   errflag = WSAGetLastError ();
#endif
   return (errflag);
}




pop3_socket::pop3_socket ()
{
   encryptable = false;
   encryptit = true;
   tv.tv_sec = 5;
   tv.tv_usec = 0;
   
}

pop3_socket::~pop3_socket ()
{
}

bool pop3_socket::open (char *host)
{
   bool retval = true;
   int cnt;
   SOCKET solist[1];
   bool soready[1];
   struct sockaddr_in sain;
   struct sockaddr *psain;
   struct hostent *hst;
   int port = 110;
   char *ptr;
   char *ptr2;
   
   try
   {
      encryptable = false;
      retval = sock.create (so, AF_INET, SOCK_STREAM, 0);
      if (retval == false)
         throw CPPSOCKET_ERROR;
// Set host.
      sain.sin_addr.s_addr = inet_addr (host);
      if (sain.sin_addr.s_addr == CPPSOCKET_ERROR)
      {
         hst = gethostbyname (host);
         if (hst != 0)
         {
            sain.sin_addr.s_addr = *((unsigned long *) hst->h_addr);
         }
      }
      if (sain.sin_addr.s_addr == CPPSOCKET_ERROR)
         throw CPPSOCKET_ERROR;
// Set port.
      sain.sin_port = htons ((u_short) port);
// Set domain.     
      sain.sin_family = AF_INET;

      psain = (struct sockaddr *) &sain;
      retval = sock.connect (so, *psain);
      if (retval == false)
         throw CPPSOCKET_ERROR;
      solist[0] = so;
      if (sock.isreadready (soready, solist, 1, &tv) <= 0)
         throw CPPSOCKET_TIMEOUT;
      memset ((void *) &buffer, '\0', CPPSOCKET_BUFFER_SIZE);
      cnt = sock.receive (so, buffer, CPPSOCKET_BUFFER_SIZE);
      if (strncmp (buffer, "+OK", 3) != 0)
         throw CPPSOCKET_NAK;
      ptr = strchr (buffer, '<');
      if (ptr != 0)
      {
         encryptable = true;
         ptr2 = strchr (ptr, '>');
         ptr2++;
         *ptr2 = '\0';
         strcpy (encservaddr, ptr);
      }
      else
         encservaddr[0] = '\0';
   }
   catch (int errflag)
   {
      retval = false;
   }
   return (retval);   
}

bool pop3_socket::isencryptable ()
{
   return (encryptable);
}

void pop3_socket::setencrypt (bool flag)
{
   encryptit = flag;
}

bool pop3_socket::login (char *user, char *password)
{
   bool retval = false;

   if ((encryptit == true) && (encryptable == true))
      retval = apop (user, password);      
   if (encryptit == false)
      retval = userpassword (user, password);      
   return (retval);
}

bool pop3_socket::userpassword (char *user, char *password)
{
   bool retval = true;
   int cnt;
   SOCKET solist[1];
   bool soready[1];

   try
   {
      memset ((void *) &buffer, '\0', CPPSOCKET_BUFFER_SIZE);
      strcpy (buffer, "USER ");
      if (strlen (user) + 7 > CPPSOCKET_BUFFER_SIZE)
         throw CPPSOCKET_BUFFER_OVERFLOW;
      strcat (buffer, user);
      strcat (buffer, "\r\n");
      cnt = sock.send (so, buffer);
      if (cnt == CPPSOCKET_ERROR)
         throw CPPSOCKET_ERROR;
      solist[0] = so;   
      if (sock.isreadready (soready, solist, 1, &tv) <= 0)
         throw CPPSOCKET_TIMEOUT;
      memset ((void *) &buffer, '\0', CPPSOCKET_BUFFER_SIZE);
      cnt = sock.receive (so, buffer, CPPSOCKET_BUFFER_SIZE);
      if (cnt == CPPSOCKET_ERROR)
         throw CPPSOCKET_ERROR;
      if (strncmp (buffer, "+OK", 3) != 0)
         throw CPPSOCKET_NAK;
         
      memset ((void *) &buffer, '\0', CPPSOCKET_BUFFER_SIZE);
      strcpy (buffer, "PASS ");
      if (strlen (password) + 7 > CPPSOCKET_BUFFER_SIZE)
         throw CPPSOCKET_BUFFER_OVERFLOW;
      strcat (buffer, password);
      strcat (buffer, "\r\n");
      cnt = sock.send (so, buffer);
      if (cnt == CPPSOCKET_ERROR)
         throw CPPSOCKET_ERROR;
      if (sock.isreadready (soready, solist, 1, &tv) <= 0)
         throw CPPSOCKET_TIMEOUT;
      memset ((void *) &buffer, '\0', CPPSOCKET_BUFFER_SIZE);
      cnt = sock.receive (so, buffer, CPPSOCKET_BUFFER_SIZE);
      if (cnt == CPPSOCKET_ERROR)
         throw CPPSOCKET_ERROR;
      if (strncmp (buffer, "+OK", 3) != 0)
         throw CPPSOCKET_NAK;
   }
   catch (int errflag)
   {
      retval = false;
   }
   return (retval);
}

bool pop3_socket::quit (void)
{
   bool retval = true;
   int cnt;
   try
   {
      memset ((void *) &buffer, '\0', CPPSOCKET_BUFFER_SIZE);
      strcpy (buffer, "QUIT");
      strcat (buffer, "\r\n");
      cnt = sock.send (so, buffer);
      if (cnt == CPPSOCKET_ERROR)
         throw CPPSOCKET_ERROR;
   }
   catch (int errflag)
   {
      retval = false;
   }
   return (retval);  
}

bool pop3_socket::close (void)
{
   return (sock.close (so));   
}

int pop3_socket::stat (void)
{
   int cnt;
   SOCKET solist[1];
   bool soready[1];
   int sizecnt;
   char *ptr;

   try
   {
      memset ((void *) &buffer, '\0', CPPSOCKET_BUFFER_SIZE);
      strcpy (buffer, "STAT");
      strcat (buffer, "\r\n");
      cnt = sock.send (so, buffer);
      if (cnt == CPPSOCKET_ERROR)
         throw CPPSOCKET_ERROR;
      solist[0] = so;   
      if (sock.isreadready (soready, solist, 1, &tv) <= 0)
         throw CPPSOCKET_TIMEOUT;
      memset ((void *) &buffer, '\0', CPPSOCKET_BUFFER_SIZE);
      cnt = sock.receive (so, buffer, CPPSOCKET_BUFFER_SIZE);
      if (cnt == CPPSOCKET_ERROR)
         throw CPPSOCKET_ERROR;
      ptr = strtok (buffer, " ");
      if (ptr != 0)
         ptr = strtok (0, " ");
      sizecnt = atoi (ptr);   
   }
   catch (int errflag)
   {
      sizecnt = 0;
   }
   return (sizecnt);
}

bool pop3_socket::noop (void)
{
   bool retval = true;
   int cnt;
   SOCKET solist[1];
   bool soready[1];

   try
   {
      memset ((void *) &buffer, '\0', CPPSOCKET_BUFFER_SIZE);
      strcpy (buffer, "NOOP");
      strcat (buffer, "\r\n");
      cnt = sock.send (so, buffer);
      if (cnt == CPPSOCKET_ERROR)
         throw CPPSOCKET_ERROR;
      solist[0] = so;   
      if (sock.isreadready (soready, solist, 1, &tv) <= 0)
         throw CPPSOCKET_TIMEOUT;
      memset ((void *) &buffer, '\0', CPPSOCKET_BUFFER_SIZE);
      cnt = sock.receive (so, buffer, CPPSOCKET_BUFFER_SIZE);
      if (cnt == CPPSOCKET_ERROR)
         throw CPPSOCKET_ERROR;
      if (strncmp (buffer, "+OK", 3) != 0)
         throw CPPSOCKET_NAK;
   }
   catch (int errflag)
   {
      retval = false;
   }
   return (retval);  
}

bool pop3_socket::rset (void)
{
   bool retval = true;
   int cnt;
   SOCKET solist[1];
   bool soready[1];
   
   try
   {
      memset ((void *) &buffer, '\0', CPPSOCKET_BUFFER_SIZE);
      strcpy (buffer, "RSET");
      strcat (buffer, "\r\n");
      cnt = sock.send (so, buffer);
      if (cnt == CPPSOCKET_ERROR)
         throw CPPSOCKET_ERROR;
      solist[0] = so;   
      if (sock.isreadready (soready, solist, 1, &tv) <= 0)
         throw CPPSOCKET_TIMEOUT;
      memset ((void *) &buffer, '\0', CPPSOCKET_BUFFER_SIZE);
      cnt = sock.receive (so, buffer, CPPSOCKET_BUFFER_SIZE);
      if (cnt == CPPSOCKET_ERROR)
         throw CPPSOCKET_ERROR;
      if (strncmp (buffer, "+OK", 3) != 0)
         throw CPPSOCKET_NAK;
   }
   catch (int errflag)
   {
      retval = false;
   }
   return (retval);   
}

bool pop3_socket::uidl (char *msg, int lnth, int msgnum)
{
   bool retval = true;
   int cnt;
   int id = CPPSOCKET_ERROR;
   SOCKET solist[1];
   bool soready[1];
   char *ptr;

   try
   {
      memset ((void *) &buffer, '\0', CPPSOCKET_BUFFER_SIZE);
      strcpy (buffer, "UIDL");
      strcat (buffer, " ");
      itoa (msgnum, &buffer[5], 10);
      strcat (buffer, "\r\n");
      cnt = sock.send (so, buffer);
      if (cnt == CPPSOCKET_ERROR)
         throw CPPSOCKET_ERROR;
      solist[0] = so;   
      if (sock.isreadready (soready, solist, 1, &tv) <= 0)
         throw CPPSOCKET_TIMEOUT;
      memset ((void *) &buffer, '\0', CPPSOCKET_BUFFER_SIZE);
      cnt = sock.receive (so, buffer, CPPSOCKET_BUFFER_SIZE);
      if (cnt == CPPSOCKET_ERROR)
         throw CPPSOCKET_ERROR;
      if (strncmp (buffer, "+OK", 3) != 0)
         throw CPPSOCKET_NAK;
      ptr = strtok (buffer, " ");
      if (ptr != 0)
         ptr = strtok (0, " ");
      if (ptr != 0)
         ptr = strtok (0, " ");
      cnt = strlen (ptr);
      if (cnt > lnth)
         cnt = lnth;
      strncpy (msg, ptr, cnt);
      msg[lnth-1] = '\0';
   }
   catch (int errflag)
   {
      retval = false;
      msg[0] = '\0';
   }
   return (retval);   
}

int pop3_socket::list (int msgnum)
{
   bool retval = true;
   int cnt;
   int sizecnt = CPPSOCKET_ERROR;
   SOCKET solist[1];
   bool soready[1];
   char *ptr;
   char *ptr2;

   try
   {
      memset ((void *) &buffer, '\0', CPPSOCKET_BUFFER_SIZE);
      strcpy (buffer, "LIST");
      strcat (buffer, " ");
      itoa (msgnum, &buffer[5], 10);
      strcat (buffer, "\r\n");
      cnt = sock.send (so, buffer);
      if (cnt == CPPSOCKET_ERROR)
         throw CPPSOCKET_ERROR;
      solist[0] = so;   
      if (sock.isreadready (soready, solist, 1, &tv) <= 0)
         throw CPPSOCKET_TIMEOUT;
      memset ((void *) &buffer, '\0', CPPSOCKET_BUFFER_SIZE);
      cnt = sock.receive (so, buffer, CPPSOCKET_BUFFER_SIZE);
      if (cnt == CPPSOCKET_ERROR)
         throw CPPSOCKET_ERROR;
      if (strncmp (buffer, "+OK", 3) != 0)
         throw CPPSOCKET_NAK;
      ptr = strtok (buffer, " ");
      if (ptr != 0)
         ptr = strtok (0, " ");
      if (ptr != 0)
         ptr = strtok (0, " ");
      sizecnt = atoi (ptr);
   }
   catch (int errflag)
   {
      sizecnt = CPPSOCKET_ERROR;
   }
   return (sizecnt);
   
}

int pop3_socket::retr (char *msg, int lnth, int msgnum)
{
   int retval = 0;
   int cnt;
   int id = CPPSOCKET_ERROR;
   SOCKET solist[1];
   bool soready[1];
   int totcnt = 0;
   bool done = false;
   int x;
   int lnth2;
   char *ptr;
   char *ptr2 = 0;
   
   try
   {
      memset ((void *) &buffer, '\0', CPPSOCKET_BUFFER_SIZE);
      strcpy (buffer, "RETR ");
      itoa (msgnum, &buffer[5], 10);
      strcat (buffer, "\r\n");
      cnt = sock.send (so, buffer);
      if (cnt == CPPSOCKET_ERROR)
         throw CPPSOCKET_ERROR;
      solist[0] = so;
      if (sock.isreadready (soready, solist, 1, &tv) <= 0)
         throw CPPSOCKET_TIMEOUT;
      memset ((void *) &buffer, '\0', CPPSOCKET_BUFFER_SIZE);
      cnt = sock.receive (so, buffer, CPPSOCKET_BUFFER_SIZE);
      if (cnt == CPPSOCKET_ERROR)
         throw CPPSOCKET_ERROR;
      if (strncmp (buffer, "+OK", 3) != 0)
         throw CPPSOCKET_NAK;
      if (cnt < CPPSOCKET_BUFFER_SIZE)
         lnth2 = cnt;
      else 
         lnth2 = CPPSOCKET_BUFFER_SIZE - 1;
      buffer[lnth2] = '\0';
      ptr = strstr (buffer, "\r\n");
      ptr++;
      ptr++;
      while (ptr2 == 0)
      {
         ptr2 = strstr (ptr, "\r\n.");
         if (ptr2 != 0)
         {
            ptr2++;
            ptr2++;
            *ptr2 = '\0';
         }         
         else
         {
            lnth2 = strlen (buffer);
            if (lnth2 == CPPSOCKET_BUFFER_SIZE)
               ptr2 = ptr;
            else   
               cnt = sock.receive (so, &buffer[lnth2], CPPSOCKET_BUFFER_SIZE - lnth2);
         }
      }
      lnth2 = strlen (ptr);
      if (lnth2 > lnth)
         lnth2 = lnth;
      strncpy (msg, ptr, lnth2);
      msg[lnth2 - 1] = '\0';
   }
   catch (int errflag)
   {
      retval = errflag;
      msg[0] = '\0';
   }
   return (retval);   
}

bool pop3_socket::dele (int msgnum)
{
   bool retval = true;
   int cnt;
   SOCKET solist[1];
   bool soready[1];
   
   try
   {
      memset ((void *) &buffer, '\0', CPPSOCKET_BUFFER_SIZE);
      strcpy (buffer, "DELE ");
      itoa (msgnum, &buffer[5], 10);
      strcat (buffer, "\r\n");
      cnt = sock.send (so, buffer);
      if (cnt == CPPSOCKET_ERROR)
         throw CPPSOCKET_ERROR;
      solist[0] = so;
      if (sock.isreadready (soready, solist, 1, &tv) <= 0)
         throw CPPSOCKET_TIMEOUT;
      memset ((void *) &buffer, '\0', CPPSOCKET_BUFFER_SIZE);
      cnt = sock.receive (so, buffer, CPPSOCKET_BUFFER_SIZE);
      if (cnt == CPPSOCKET_ERROR)
         throw CPPSOCKET_ERROR;
      if (strncmp (buffer, "+OK", 3) != 0)
         throw CPPSOCKET_NAK;
   }
   catch (int errflag)
   {
      retval = false;
   }
   return (retval);   
}

bool pop3_socket::apop (char *user, char *password)
{
   bool retval = true;
   int cnt;
   char buf[CPPSOCKET_ENCRYPTION_BUFFER_SIZE];
   SOCKET solist[1];
   bool soready[1];
   
   try
   {
      strcpy (buffer, encservaddr);
      strcat (buffer, password);
      buf[0] = '\0';
// Replace with call to MD5 function.      
//    encode_md5 (buf, buffer);
      memset ((void *) &buffer, '\0', CPPSOCKET_BUFFER_SIZE);
      strcpy (buffer, "APOP ");
      strcat (buffer, user);
      strcat (buffer, " ");
      strcat (buffer, buf);
      strcat (buffer, "\r\n");
      cnt = sock.send (so, buffer);
      if (cnt == CPPSOCKET_ERROR)
         throw CPPSOCKET_ERROR;
      solist[0] = so;
      if (sock.isreadready (soready, solist, 1, &tv) <= 0)
         throw CPPSOCKET_TIMEOUT;
      memset ((void *) &buffer, '\0', CPPSOCKET_BUFFER_SIZE);
      cnt = sock.receive (so, buffer, CPPSOCKET_BUFFER_SIZE);
      if (cnt == CPPSOCKET_ERROR)
         throw CPPSOCKET_ERROR;
      if (strncmp (buffer, "+OK", 3) != 0)
         throw CPPSOCKET_NAK;
   }
   catch (int errflag)
   {
      retval = false;
   }
   return (retval);   
}


void do_pop3 ()
{
   bool retval;
   char buf[60000];
   pop3_socket mailstream;
   retval = mailstream.open ("your.pop3.address.org");
   mailstream.setencrypt (false);
   retval = mailstream.login ("yourusername", "yourpassword");
   int cnt = mailstream.stat ();
   cout << cnt << " message(s)" << endl;
   cout <<  mailstream.quit ()  << endl;
   retval = mailstream.close ();
}


main ()
{
   do_pop3 ();
}

